#include "poly_FIR_dec.hxx"

poly_FIR_dec::poly_FIR_dec() {}


poly_FIR_dec::poly_FIR_dec(const std::vector<CLAM::TData> &filtCoeff, int filterSize)
{

    _filterBufferOdd = boost::circular_buffer<CLAM::TData>(filterSize/2,0.0);
    _filterOdd = std::vector<CLAM::TData>(filterSize/2,0.0);

    if ( (filterSize%2) == 1) {
       	_filterBufferEven = boost::circular_buffer<CLAM::TData>(filterSize/2 + 1, 0.0);
      	_filterEven = std::vector<CLAM::TData>(filterSize/2 + 1, 0.0);
    }
    else {
      	_filterBufferEven = boost::circular_buffer<CLAM::TData>(filterSize/2, 0.0);
    	_filterEven = std::vector<CLAM::TData>(filterSize/2, 0.0);
    }

    //Below the polyphase filters are created

    for (std::vector<CLAM::TData>::size_type i1=0; i1 < _filterBufferEven.size(); ++i1) {
        _filterEven[i1] = filtCoeff[2*i1];
    }

    for (std::vector<CLAM::TData>::size_type i2=0; i2 < _filterBufferOdd.size(); ++i2) {
        _filterOdd[i2] = filtCoeff[2*i2+1];
    }

}

void poly_FIR_dec::push_to_polyphase_even(CLAM::TData mysample)
{
    //even samples go here
    _filterBufferEven.push_front(mysample);
}

void poly_FIR_dec::push_to_polyphase_odd(CLAM::TData mysample)
{
    //odd samples go here
    _filterBufferOdd.push_front(mysample);
}

CLAM::TData poly_FIR_dec::calculate_direct()
{
    /*filter is calculated at half the input sample rate so
    make sure to evaluate this function only at half the
    samplerate*/

    CLAM::TData result1 = 0.0;
    CLAM::TData result2 = 0.0;

    vec_it filterIteratorEven = _filterEven.begin();

    for (circ_it i = _filterBufferEven.begin(); i != _filterBufferEven.end(); ++i) {
        result1 += (*filterIteratorEven) * (*i);
        ++filterIteratorEven;
    }

    vec_it filterIteratorOdd = _filterOdd.begin();

    for (circ_it i = _filterBufferOdd.begin(); i != _filterBufferOdd.end(); ++i) {
        result2 += (*filterIteratorOdd) * (*i);
        ++filterIteratorOdd;
    }

    return (result1 + result2);

}

CLAM::TData poly_FIR_dec::calculate_FFT() {
    //implement this one later
    return 0.0;
}

