/*Class to implement Polyphase FIR decimation filtering of a factor 2.
Note that this structure doesn't deal with the switching of the decimation,
that has to be dealt with in the program using poly_FIR_dec. Two samples are
pushed in to the two filter buffers in turn, then one one output sample is calculated.
Three modes for the implementation of the FIR is possible.

1) Direct Form. 2) Direct Form using linear phase symmetry
3) Fast Convolution using the FFT*/

#ifndef poly_FIR_dec_hxx
#define poly_FIR_dec_hxx

#include <boost/circular_buffer.hpp>
#include <CLAM/DataTypes.hxx>
#include<vector>

typedef boost::circular_buffer<CLAM::TData>::iterator circ_it;
typedef boost::circular_buffer<CLAM::TData>::iterator reverse_circ_it;
typedef std::vector<CLAM::TData>::iterator vec_it;

class poly_FIR_dec {

public:

    poly_FIR_dec();
    poly_FIR_dec(const std::vector<CLAM::TData> &filtCoeff, int filterSize);

    void push_to_polyphase_even(CLAM::TData mysample);

    void push_to_polyphase_odd(CLAM::TData mysample);

    CLAM::TData calculate_direct();
    CLAM::TData calculate_FFT();

private:

    boost::circular_buffer<CLAM::TData> _filterBufferEven;
    boost::circular_buffer<CLAM::TData> _filterBufferOdd;

    std::vector<CLAM::TData> _filterEven;
    std::vector<CLAM::TData> _filterOdd;

};

#endif

