#include "poly_FIR_interp.hxx"

poly_FIR_interp::poly_FIR_interp() { }

poly_FIR_interp::poly_FIR_interp(const std::vector<CLAM::TData> &filtCoeff, int filterSize) {

   //In this constructor the pholyphase coefficients from the filter are extracted

    _filterBufferOdd = boost::circular_buffer<CLAM::TData>(filterSize/2,0.0);
    _filterOdd = std::vector<CLAM::TData>(filterSize/2,0.0);

    if ( (filterSize%2) == 1) {
	_filterBufferEven = boost::circular_buffer<CLAM::TData>(filterSize/2 + 1, 0.0);
    	_filterEven = std::vector<CLAM::TData>(filterSize/2 + 1, 0.0);
    }
    else {
	_filterBufferEven = boost::circular_buffer<CLAM::TData>(filterSize/2, 0.0);
    	_filterEven = std::vector<CLAM::TData>(filterSize/2, 0.0);
    }

    //Below the polyphase filters are created

    for (std::vector<CLAM::TData>::size_type i1=0; i1 < _filterBufferEven.size(); ++i1) {
        _filterEven[i1] = filtCoeff[2*i1];
    }

    for (std::vector<CLAM::TData>::size_type i2=0; i2 < _filterBufferOdd.size(); ++i2) {
        _filterOdd[i2] = filtCoeff[2*i2+1];
    }

}

void poly_FIR_interp::push_and_calculate_direct(CLAM::TData mysample)
{
    _filterBufferEven.push_front(mysample);
    _filterBufferOdd.push_front(mysample);

    vec_it filterIteratorEven = _filterEven.begin();

    for (circ_it i = _filterBufferEven.begin(); i != _filterBufferEven.end(); ++i) {
        _output1 += (*filterIteratorEven) * (*i);
        ++filterIteratorEven;
    }

    vec_it filterIteratorOdd = _filterOdd.begin();

    for (circ_it i = _filterBufferOdd.begin(); i != _filterBufferOdd.end(); ++i) {
        _output2 += (*filterIteratorOdd) * (*i);
        ++filterIteratorOdd;
    }

}

void poly_FIR_interp::push_and_calculate_FFT(CLAM::TData mysample)
{
    _output1 = 0.0;
    _output2 = 0.0;
}

CLAM::TData poly_FIR_interp::get_first_output_sample() {
    return _output1;
}

CLAM::TData poly_FIR_interp::get_second_output_sample() {
    return _output2;
}
