/*Class to implement Polyphase FIR Interpolation of a factor 2.
You push one sample into the structure and take out two.
Three modes for the implementation of the FIR is possible:

1) Direct Form. 2) Direct Form using linear phase symmetry
3) Fast Convolution using the FFT*/

#ifndef poly_FIR_interp_hxx
#define poly_FIR_interp_hxx

#include <boost/circular_buffer.hpp>
#include <CLAM/DataTypes.hxx>
#include<vector>

typedef boost::circular_buffer<CLAM::TData>::iterator circ_it;
typedef boost::circular_buffer<CLAM::TData>::iterator reverse_circ_it;
typedef std::vector<CLAM::TData>::iterator vec_it;

class poly_FIR_interp {

public:

    poly_FIR_interp();
    poly_FIR_interp(const std::vector<CLAM::TData> &filtCoeff1, int filterSize);

    void push_and_calculate_direct(CLAM::TData mysample);
    void push_and_calculate_FFT(CLAM::TData mysample);

    CLAM::TData get_first_output_sample();
    CLAM::TData get_second_output_sample();

private:

    boost::circular_buffer<CLAM::TData> _filterBufferEven;
    boost::circular_buffer<CLAM::TData> _filterBufferOdd;
    CLAM::TData _output1;
    CLAM::TData _output2;

    std::vector<CLAM::TData> _filterEven;
    std::vector<CLAM::TData> _filterOdd;

};

#endif


